@@ -1,19 +1,20 @@
-from __future__ import unicode_literals
-
 import logging
 import sys
 import threading
 from contextlib import contextmanager
+from pathlib import Path
 from stat import S_IWGRP, S_IWOTH, S_IWUSR
+from types import TracebackType
+from typing import Callable, Iterator, Optional, Tuple, Type, Union
 
 import pytest
+from _pytest.logging import LogCaptureFixture
 
-from filelock import FileLock, SoftFileLock, Timeout
-from filelock._util import PermissionError
+from filelock import BaseFileLock, FileLock, SoftFileLock, Timeout
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_simple(lock_type, tmp_path, caplog):
+def test_simple(lock_type: Type[BaseFileLock], tmp_path: Path, caplog: LogCaptureFixture) -> None:
     caplog.set_level(logging.DEBUG)
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -24,17 +25,17 @@ def test_simple(lock_type, tmp_path, caplog):
     assert not lock.is_locked
 
     assert caplog.messages == [
-        "Attempting to acquire lock {} on {}".format(id(lock), lock_path),
-        "Lock {} acquired on {}".format(id(lock), lock_path),
-        "Attempting to release lock {} on {}".format(id(lock), lock_path),
-        "Lock {} released on {}".format(id(lock), lock_path),
+        f"Attempting to acquire lock {id(lock)} on {lock_path}",
+        f"Lock {id(lock)} acquired on {lock_path}",
+        f"Attempting to release lock {id(lock)} on {lock_path}",
+        f"Lock {id(lock)} released on {lock_path}",
     ]
     assert [r.levelno for r in caplog.records] == [logging.DEBUG, logging.DEBUG, logging.DEBUG, logging.DEBUG]
     assert [r.name for r in caplog.records] == ["filelock", "filelock", "filelock", "filelock"]
 
 
 @contextmanager
-def make_ro(path):
+def make_ro(path: Path) -> Iterator[None]:
     write = S_IWUSR | S_IWGRP | S_IWOTH
     path.chmod(path.stat().st_mode & ~write)
     yield
@@ -42,21 +43,21 @@ def make_ro(path):
 
 
 @pytest.fixture()
-def tmp_path_ro(tmp_path):
+def tmp_path_ro(tmp_path: Path) -> Iterator[Path]:
     with make_ro(tmp_path):
         yield tmp_path
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
 @pytest.mark.skipif(sys.platform == "win32", reason="Windows does not have read only folders")
-def test_ro_folder(lock_type, tmp_path_ro):
+def test_ro_folder(lock_type: Type[BaseFileLock], tmp_path_ro: Path) -> None:
     lock = lock_type(str(tmp_path_ro / "a"))
     with pytest.raises(PermissionError, match="Permission denied"):
         lock.acquire()
 
 
 @pytest.fixture()
-def tmp_file_ro(tmp_path):
+def tmp_file_ro(tmp_path: Path) -> Iterator[Path]:
     filename = tmp_path / "a"
     filename.write_text("")
     with make_ro(filename):
@@ -64,14 +65,14 @@ def tmp_file_ro(tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_ro_file(lock_type, tmp_file_ro):
+def test_ro_file(lock_type: Type[BaseFileLock], tmp_file_ro: Path) -> None:
     lock = lock_type(str(tmp_file_ro))
     with pytest.raises(PermissionError, match="Permission denied"):
         lock.acquire()
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_missing_directory(lock_type, tmp_path_ro):
+def test_missing_directory(lock_type: Type[BaseFileLock], tmp_path_ro: Path) -> None:
     lock_path = tmp_path_ro / "a" / "b"
     lock = lock_type(str(lock_path))
 
@@ -80,7 +81,7 @@ def test_missing_directory(lock_type, tmp_path_ro):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_nested_context_manager(lock_type, tmp_path):
+def test_nested_context_manager(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # lock is not released before the most outer with statement that locked the lock, is left
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -103,7 +104,7 @@ def test_nested_context_manager(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_nested_acquire(lock_type, tmp_path):
+def test_nested_acquire(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # lock is not released before the most outer with statement that locked the lock, is left
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -126,7 +127,7 @@ def test_nested_acquire(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_nested_forced_release(lock_type, tmp_path):
+def test_nested_forced_release(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # acquires the lock using a with-statement and releases the lock before leaving the with-statement
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -142,40 +143,40 @@ def test_nested_forced_release(lock_type, tmp_path):
     assert not lock.is_locked
 
 
+_ExcInfoType = Union[Tuple[Type[BaseException], BaseException, TracebackType], Tuple[None, None, None]]
+
+
 class ExThread(threading.Thread):
-    def __init__(self, target, name):
-        super(ExThread, self).__init__(target=target, name=name)
-        self.ex = None
+    def __init__(self, target: Callable[[], None], name: str) -> None:
+        super().__init__(target=target, name=name)
+        self.ex: Optional[_ExcInfoType] = None
 
-    def run(self):
+    def run(self) -> None:
         try:
-            super(ExThread, self).run()
-        except Exception:
-            self.ex = sys.exc_info()
+            super().run()
+        except Exception:  # pragma: no cover
+            self.ex = sys.exc_info()  # pragma: no cover
 
-    def join(self, timeout=None):
-        super(ExThread, self).join(timeout=timeout)
+    def join(self, timeout: Optional[float] = None) -> None:
+        super().join(timeout=timeout)
         if self.ex is not None:
-            print("fail from thread {}".format(self.name))
-            if sys.version_info[0] == 2:
-                wrapper_ex = self.ex[1]
-                raise (wrapper_ex.__class__, wrapper_ex, self.ex[2])
-            raise self.ex[0].with_traceback(self.ex[1], self.ex[2])
+            print(f"fail from thread {self.name}")  # pragma: no cover
+            raise RuntimeError from self.ex[1]  # pragma: no cover
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_threaded_shared_lock_obj(lock_type, tmp_path):
+def test_threaded_shared_lock_obj(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # Runs 100 threads, which need the filelock. The lock must be acquired if at least one thread required it and
     # released, as soon as all threads stopped.
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
 
-    def thread_work():
+    def thread_work() -> None:
         for _ in range(100):
             with lock:
                 assert lock.is_locked
 
-    threads = [ExThread(target=thread_work, name="t{}".format(i)) for i in range(100)]
+    threads = [ExThread(target=thread_work, name=f"t{i}") for i in range(100)]
     for thread in threads:
         thread.start()
     for thread in threads:
@@ -185,17 +186,18 @@ def test_threaded_shared_lock_obj(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_threaded_lock_different_lock_obj(lock_type, tmp_path):
+@pytest.mark.skipif(hasattr(sys, "pypy_version_info") and sys.platform == "win32", reason="deadlocks randomly")
+def test_threaded_lock_different_lock_obj(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # Runs multiple threads, which acquire the same lock file with a different FileLock object. When thread group 1
     # acquired the lock, thread group 2 must not hold their lock.
 
-    def t_1():
+    def t_1() -> None:
         for _ in range(1000):
             with lock_1:
                 assert lock_1.is_locked
                 assert not lock_2.is_locked
 
-    def t_2():
+    def t_2() -> None:
         for _ in range(1000):
             with lock_2:
                 assert not lock_1.is_locked
@@ -203,7 +205,7 @@ def test_threaded_lock_different_lock_obj(lock_type, tmp_path):
 
     lock_path = tmp_path / "a"
     lock_1, lock_2 = lock_type(str(lock_path)), lock_type(str(lock_path))
-    threads = [(ExThread(t_1, "t1_{}".format(i)), ExThread(t_2, "t2_{}".format(i))) for i in range(10)]
+    threads = [(ExThread(t_1, f"t1_{i}"), ExThread(t_2, f"t2_{i}")) for i in range(10)]
 
     for thread_1, thread_2 in threads:
         thread_1.start()
@@ -217,7 +219,7 @@ def test_threaded_lock_different_lock_obj(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_timeout(lock_type, tmp_path):
+def test_timeout(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # raises Timeout error when the lock cannot be acquired
     lock_path = tmp_path / "a"
     lock_1, lock_2 = lock_type(str(lock_path)), lock_type(str(lock_path))
@@ -240,7 +242,7 @@ def test_timeout(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_default_timeout(lock_type, tmp_path):
+def test_default_timeout(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # test if the default timeout parameter works
     lock_path = tmp_path / "a"
     lock_1, lock_2 = lock_type(str(lock_path)), lock_type(str(lock_path), timeout=0.1)
@@ -272,7 +274,7 @@ def test_default_timeout(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_context_release_on_exc(lock_type, tmp_path):
+def test_context_release_on_exc(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # lock is released when an exception is thrown in a with-statement
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -287,7 +289,7 @@ def test_context_release_on_exc(lock_type, tmp_path):
 
 
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_acquire_release_on_exc(lock_type, tmp_path):
+def test_acquire_release_on_exc(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # lock is released when an exception is thrown in a acquire statement
     lock_path = tmp_path / "a"
     lock = lock_type(str(lock_path))
@@ -303,7 +305,7 @@ def test_acquire_release_on_exc(lock_type, tmp_path):
 
 @pytest.mark.skipif(hasattr(sys, "pypy_version_info"), reason="del() does not trigger GC in PyPy")
 @pytest.mark.parametrize("lock_type", [FileLock, SoftFileLock])
-def test_del(lock_type, tmp_path):
+def test_del(lock_type: Type[BaseFileLock], tmp_path: Path) -> None:
     # lock is released when the object is deleted
     lock_path = tmp_path / "a"
     lock_1, lock_2 = lock_type(str(lock_path)), lock_type(str(lock_path))
@@ -326,7 +328,7 @@ def test_del(lock_type, tmp_path):
     lock_2.release()
 
 
-def test_cleanup_soft_lock(tmp_path):
+def test_cleanup_soft_lock(tmp_path: Path) -> None:
     # tests if the lock file is removed after use
     lock_path = tmp_path / "a"
     lock = SoftFileLock(str(lock_path))
